/* Copyright 2011, 2012 Kevin Ryde

   This file is part of Image-Base-GD.

   Image-Base-GD is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 3, or (at your option) any later
   version.

   Image-Base-GD is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License along
   with Image-Base-GD.  If not, see <http://www.gnu.org/licenses/>.  */

#include <gd.h>
#include <stdio.h>

int
main (void)
{
  gdImagePtr gd;
  FILE *out;
  gd = gdImageCreate(13,17);
  out = fopen ("/tmp/GD-gd1.gd", "w");
  gdImageGd (gd, out);
  return 0;
}
